// This is our javascript, which will pop up our message
// in an alert box.

var Debug;
var DebugMode = 1;
var OtherWin;
var domain;
var RefURI;
var Store = {}; // this is where we'll store all the data
const notWhitespace = /\S/;

/***************************************************************************
 GetURI: returns the URI in the current browser window, determines the 
well-known location of the policy reference file, and calls importXML on it.
****************************************************************************/
function GetURI(){
  
  Store = {}; // delete previously stored data
  if (DebugMode) {
    var StatsTab = gBrowser.addTab("");
    Debug = gBrowser.getBrowserForTab(StatsTab).contentDocument;
    Debug.write("<title>Debugging output for Privacy Fox</title>");
  }

  var uriToValidate = getBrowser().contentDocument.location;
  //  alert("URI in browser window is " + uriToValidate);
  var regexpURI = /:\/\/((\w+\.?)+)\//;
  if (regexpURI.test(uriToValidate)) {
    Notify("The server is " + RegExp.$1);
  }
  else {
    alert("Privacy Fox could not obtain the current server's name. Sorry.");
    Debug.close();
    return(-1);
  }
  RefURI = "http://" + RegExp.$1 + "/w3c/p3p.xml";
  //Notify("Calling importXML to parse the policy reference file: " + RefURI);
  domain =  RegExp.$1;
  // Notify("Global var domain set to " + domain);
  importXML(RefURI, 1);   
  /*  
      Notify("Calling FetchXML to parse the policy reference file: " + RefURI);
      var txt = FetchXML2(RefURI);
      Notify("FetchXML returned " + txt);
      Debug.write(txt);
  */
}
/***************************************************************************
FetchXML2: another try at getting xmlhttp to work
****************************************************************************/
function FetchXML2(targetURL) {
  targetURL = "http://p3p.yahoo.com/w3c/p3p.xml";
  var xmlhttp=false;
  xmlhttp = new XMLHttpRequest();
  Notify("Sending GET request in FetchXML2() to " + targetURL);
  xmlhttp.open("GET", targetURL,true);
  xmlhttp.onreadystatechange=function() {
    if (xmlhttp.readyState==4) {
      Notify(xmlhttp.responseText);
    }
  }
  Notify("Before send call.");
  xmlhttp.send(null);
  Notify("After send call.");
  return xmlhttp.responseText;

}

/***************************************************************************
FetchXML: an attempt at getting xmlhttprequest to work
****************************************************************************/
function FetchXML(targetURL) {
  Notify("**Trying to use XMLHttpRequest on " + targetURL);
  //Set the server to a local variable
  // var targetURL = this.server;
  
  //Try to create our XMLHttpRequest Object
  try {
    var httpRequest = new XMLHttpRequest();
  } catch (e){
    alert('Error creating the connection to ' + targetURL + '!' );
    return;
  }
  Notify("Created new request.");
  
  //Make the connection and send our data
  try {
    /*    var txt = "?1";
	  for(var i in this.parms) {
	  txt = txt+'&'+this.parms[i].name+'='+this.parms[i].value;
	  }
    */
    //Two options here, only uncomment one of these
    //GET REQUEST
    httpRequest.open("GET", targetURL, false, null, null);  
    Notify("Opened a GET request to " + targetURL);
    //POST REQUEST EXAMPLE
    /*
    httpRequest.open("POST", targetURL+txt, false, null, null);  
    httpRequest.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
    */
    httpRequest.send('');
    Notify("GET sent");
  } catch (e){
    alert('An error has occured calling the external site: '+e);
    return false;
  }
  Notify("There was no error with the GET");
  
  //Make sure we received a valid response
  switch(httpRequest.readyState) {
  case 1,2,3:
    alert('Bad Ready State: '+httpRequest.status);
    return false;
    break;
  case 4:
    if(httpRequest.status !=200) {
      alert('The server respond with a bad status code: '+httpRequest.status);
      return false;
    } else {
      var response = httpRequest.responseText;
    }
    break;
  }
  Notify("Returning response: " + response);
  return response;
}


/***************************************************************************
  Notify: creates a <p> element at the id writeroot in the current document
  and writes the passed in msg into it.
****************************************************************************/
function Notify(msg) {
  /*  var out = document.getElementById("writeroot");
      var outP = document.createElement("p");
      var outMsg = document.createTextNode(msg);
      out.appendChild(outP).appendChild(outMsg);
  */
  // alert(msg);
  if (DebugMode)
    Debug.write("<br>" + msg + "\n");
  // dump(msg + "\n   ");
}
/***************************************************************************
importXML loads the XML doc (reference or policy file) for access via DOM
****************************************************************************/
function importXML(file, flag) {

  // Notify("The well-known location is " + file);

  if (document.implementation && document.implementation.createDocument) {
    xmlDoc = document.implementation.createDocument("", "", null);
    Notify("Inside importXML(), loading " + file);
    if (flag == 1) { // function was called by user; file is a P3P reference file
      xmlDoc.onload = LoadRefFile;
      Notify("LoadRefFile queued to be called,  domain being " +  domain);
    }
    else if (flag == 2) { // function was called internally; file is a P3P policy file
      // file = "p3p_us.xml";
      // file = "tcpulse.xml";
      xmlDoc.onload = ParsePolicyFile;      
      Notify("ParsePolicyFile queued to be called.");
    }
    xmlDoc.load(file);
    Notify(file + " loaded.");
  }
  else {
    alert('Your browser can\'t handle this script');
    return;
  }	
}


/***************************************************************************
LoadRefFile: loads the policy reference file as determined by GetURI to obtain the 
location of the policy file. This may be relative to the domain or be included in the 
reference file itself. Calls importXML with flag 2 to load the policy file.
****************************************************************************/
function LoadRefFile() {

  // alert("");
  Notify("Now inside LoadRefFile, with domain " + domain);

  // ASSMUE: There should only ever by one policy-references tag in a ref file. 
  // ASSUME: Reference file's XML tags are in uppercase.
  // ASSUME: policy ref file exists in well-known loacation (no graceful exit if it doesn't).
  var obj = xmlDoc.getElementsByTagName("POLICY-REFERENCES")[0];
  //	Notify("childNodes length before whitespace removal is " + obj.childNodes.length);
  // remove whitespaces from DOM so that they don't count as nodes
  if (!obj) {
    Notify("In LoadRefFile, I couldn't find the POLICY-REFERENCES tag.");
    alert("Privacy Fox error:\nI can't seem to parse this site's policy reference file. Sorry!");
    Debug.close();
    return -1;
  }
  for (i=0; i<obj.childNodes.length; i++) {
    if ((obj.childNodes[i].nodeType == 3) && (!notWhitespace.test(obj.childNodes[i].nodeValue))) {
      obj.removeChild(obj.childNodes[i]);
      i--;
    }
  }
  //	Notify("ChildNodes length after whitespace removal is " + obj.childNodes.length);

  //  for (i=0; i<obj.childNodes.length; i++) {	
  // var str = obj.childNodes[i].firstChild.nodeValue;
  // Notify("The nodeValue of Policy-references' first child (policy-ref) is: " + str);
  // var policy = obj.childNodes[i].getAttribute("about");
  var polRefObj = obj.getElementsByTagName("POLICY-REF")[0]; // The first POLICY tag
  if (!polRefObj) {
     Notify("In LoadRefFile, I couldn't find the POLICY-REF tag.");
     alert("Privacy Fox error:\nI can't seem to parse this site's policy reference file. Sorry!");
     Debug.close();
     return -1;
  }   
  var policy = polRefObj.getAttribute("about");
  if (!policy) {
     Notify("In LoadRefFile, I couldn't find the about attribute of the POLICY-REF tag.");
     alert("Privacy Fox error:\nI can't seem to parse this site's policy reference file. Sorry!");
     Debug.close();
     return -1;
  }
  Notify("The policy file URI/name parsed from the reference file is " + policy);
  // policy name could be absolute or relative. Handle the relative (bravenet.net)
  var regexpURI = /http:/;
  if (regexpURI.test(policy)) {
    Notify("The policy URI is absolute so domain adding is not needed: " + policy);
  }
  else {
    var regexpRel = /^\//;      
    var regexpInc = /^#/;
    if (regexpRel.test(policy)) {
      policy = "http://" + domain + policy;      
      Notify("The policy was relative to domain so domain pre-pended:  " + policy);
    }
    else if (regexpInc.test(policy)) {
      policy = RefURI + policy;      
      Notify("The policy was included in reference file so ref file pre-pended:  " + policy);
    }
    else {
      Notify("In LoadRefFile, I couldn't regexp the policy file as anticipated: " + policy);
      alert("Privacy Fox error:\nI can't seem to parse this site's policy reference file. Sorry!");
      Debug.close();
      return -1;
    }
  }
  importXML(policy, 2);

}

/***************************************************************************
  ParsePolicyFile: parses the policy file into an hash called Store{}. Calls Translate() at the end.
****************************************************************************/
function ParsePolicyFile(){

  Notify("ParsePolicyFile called.");
  // BUILD: EXPIRY tag is checked
  var obj = xmlDoc.getElementsByTagName("POLICIES")[0]; // First policies tag; there's only supposed to be 1
  if (obj) {
    Notify(" Found the POLICIES tag..");    
    for (i=0; i<obj.childNodes.length; i++) { // remove whitespace so it doesn't get treated as nodes
      if ((obj.childNodes[i].nodeType == 3) && (!notWhitespace.test(obj.childNodes[i].nodeValue))) {
	obj.removeChild(obj.childNodes[i]);
	i--;
      }
    }
  }
  else {
    Notify("<font color=\"red\">My call to getElementsByTagName failed for tag POLICIES.</font>");
    alert("Privacy Fox error:\nI can't seem to parse this site's policy. Sorry!");
    return -1;
  }
  // BUILD: Multiple <POLICY> tags to be handled
  //if (obj.childNodes.length > 1) { // if there are multiple <POLICY>  tags
  // Notify("There are multiple <POLICY>s in this policy file.");
  // expiry or policy tag can count here, so the above line is wrong.
  //}
  // BUILD: EXPIRY attribute is checked.
  // ASSMUE: DATASCHEMA and lang attributes can be safely ignored
  //	var polObj = obj.childNodes[0]; // The first POLICY tag
  var polObj = obj.getElementsByTagName("POLICY")[0]; // The first POLICY tag
  Store.policyName = polObj.getAttribute("name");
  Store.discURI = polObj.getAttribute("discuri");
  Store.optURI = polObj.getAttribute("opturi");
  var out = "The policy";
  if (Store.policyName == null) {
    Notify("WARNING: the policy has no name.");
  }
  else {
    out += " \"" + Store.policyName + "\"";
  }
  if (Store.discURI == null) {
    Notify("WARNING: the discURI attribute is missing.");
    out += " has no human-readable version";
  }
  else {
    out += " has a human-readable version at " + Store.discURI;
  }
  if (Store.optURI != null) {
    out += " and the opt-out information is available at <" + Store.optURI + ">";
  }
  Notify(out + ".");
	
  // Parsing <TEST>
  tstObj = polObj.getElementsByTagName("TEST")[0];
  if (tstObj) {
    Notify("This is a test policy.");
    Store.TEST = 1;
  }
	
  // Parsing <ENTITY>
  entObj = polObj.getElementsByTagName("ENTITY")[0];
  if (entObj) {
    //		Notify("I did find the contact information for this website.");
    var entDGrp = entObj.getElementsByTagName("DATA-GROUP")[0];
    if (entDGrp) {
      for (i=0; i<entDGrp.childNodes.length; i++) {
	if (entDGrp.childNodes[i].nodeType != 3) {
	  var type = entDGrp.childNodes[i].getAttribute("ref");
	  var typeStr = type.substr(1,type.length-1);
	  Store[typeStr] = entDGrp.childNodes[i].firstChild.nodeValue;
	  //					Notify("<" + entDGrp.childNodes[i].tagName + ">" + typeStr + " : " + Store[typeStr] );
	}
      }
    }
    else {
      Notify("I couldn't find the DATA-GROUP tag in ENTITY");
    }	
  }
  else {
    Notify("WARNING: No contact information was found for this policy!");
  }

  // Parse <ACCESS>
  var accObj = polObj.getElementsByTagName("ACCESS")[0];
  if (accObj) {
    if (accObj.childNodes.length > 0) {
      Store.ACCESS = 1;
    }
    for (i = 0 ; i < accObj.childNodes.length; i++) {
      if (accObj.childNodes[i].tagName) {
	Store["ACCESS-" + accObj.childNodes[i].tagName] = 1;
      }
    }
  }
  else {
    Notify("WARNING: I couldn't find an ACCESS tag!");
  }

  // Parse DISPUTES-GROUP. DISPUTES resol. can be of 4 types: service, indep, court, & law
  // Is there always only one DISPUTES-GROUP
  var dispGrpObj = polObj.getElementsByTagName("DISPUTES-GROUP")[0];
  if (!dispGrpObj) {
    Notify("WARNING: I couldn't find an <DISPUTES-GROUP> tag!");
    Store["DISPUTES-GROUP"] = 0;
  }
  else {
    var dispGrpArr = dispGrpObj.childNodes;
    Store["DISPUTES-GROUP"] = 1;
    // Notify("The # of children of dispute-group is " + dispGrpArr.length);
    for (i=0; i<dispGrpArr.length; i++) {
      if (dispGrpArr[i].tagName) {
	var key = "DISPUTES-" + dispGrpArr[i].getAttribute("resolution-type");
	Store[key] = 1;
	// key += "-" + "service";
	Store[key + "-service"] = dispGrpArr[i].getAttribute("service");
	Store[key + "-sDesc"] = dispGrpArr[i].getAttribute("short-description");
	if (dispGrpArr[i].getAttribute("verification")) {
	  Store[key + "-verif"] = dispGrpArr[i].getAttribute("verification");
	}
	var ldObj = dispGrpArr[i].getElementsByTagName("LONG-DESCRIPTION")[0];
	if (ldObj) {
	  // Notify("The long description is [" + ldObj.firstChild.nodeValue + "]");
	  Store[key + "-lDesc"] = ldObj.firstChild.nodeValue;
	}
	var remObj = dispGrpArr[i].getElementsByTagName("REMEDIES")[0];
	if (remObj) {
	  if (remObj.getElementsByTagName("correct")[0]) {
	    // Notify("Correct was available as one of the remedies.");
	    Store[key + "-rem-correct"] = 1;
	  }
	  if (remObj.getElementsByTagName("money")[0]) {
	    // Notify("Money was available as one of the remedies.");
	    Store[key + "-rem-money"] = 1;
	  }
	  if (remObj.getElementsByTagName("law")[0]) {
	    // Notify("Law was available as one of the remedies.");
	    Store[key + "-rem-law"] = 1;
	  }
	}		
	
      } // end if tagname
    } // end for 
  } // end else -> if dispGrpObj
	
  // Get the statements
  var stmtArr = polObj.getElementsByTagName("STATEMENT");
  Notify("The number of statements in this policy is " + stmtArr.length);
  Store["numST"] = stmtArr.length;

  // Parsing individual statements 
  for (s = 0; s < stmtArr.length; s++) {
    Notify("Parsing statement # " + s + ".");
    // var stmtObj = stmtArr[s];

    // Parse CONSEQUENCE (optional)
    var cnsqObj = stmtArr[s].getElementsByTagName("CONSEQUENCE")[0];
    if (cnsqObj) { // CONSEQUENCE is optional
      Store["ST" + s + ".CONSEQUENCE"] = cnsqObj.firstChild.nodeValue;
    }
    else {
      Notify("This statement had no CONSEQUENCE tag (it is optional).");
    }

    // Parse NON-IDENTIFIABLE (optional)
    var nonident = stmtArr[s].getElementsByTagName("NON-IDENTIFIABLE")[0];
    if (nonident) {
      Store["ST" + s + ".NON-IDENTIFIABLE"] = 1;
    }
    //   else { // the rest of the statement matters only if no non-ident claim is made

    // Parse PURPOSE
    var prpsObj = stmtArr[s].getElementsByTagName("PURPOSE")[0];
    if (prpsObj) {
      var prpsArr = prpsObj.childNodes;
      if (prpsArr.length > 0) {
	Store["ST" + s + ".PURPOSE"] = 1;
      }
      for (var i = 0; i < prpsArr.length; i++) {
	if (prpsArr[i].tagName) {
	  //	Notify("Purpose: <" + prpsObjArr[i].tagName + ">");	  
	  var req = prpsArr[i].getAttribute("required");
	  if (req) {
	    Store["ST" + s + ".PURPOSE-" + prpsArr[i].tagName] = req;
	  }
	  else {
	    Store["ST" + s + ".PURPOSE-" + prpsArr[i].tagName] = 1;
	  }
	  if (prpsArr[i].tagName == 'other-purpose') {
	    Store["ST" + s + ".other-purpose"] = prpsArr[i].nodeValue;
	  }
	}
      }
    }
    else {
      Store["ST" + s + ".PURPOSE"] = 0;
    }   

    // Parse RECEPIENT
    var rcptObj = stmtArr[s].getElementsByTagName("RECIPIENT")[0];
    if (rcptObj) {
      var rcptArr = rcptObj.childNodes;
      if ( rcptArr.length > 0) {
	Store["ST" + s + ".RECEPIENT"] = 1;
      }
      for (var i = 0; i < rcptArr.length; i++) {
	if (rcptArr[i].tagName) {
	  //	Notify("Purpose: <" + prpsObjArr[i].tagName + ">");
	  Store["ST" + s + ".RECEPIENT-" + rcptArr[i].tagName] = 1;
	}
      }
    }
    else {
      Store["ST" + s + ".RECEPIENT"] = 0;
    }      
    
    // Parse RETENTION
    var rtnObj = stmtArr[s].getElementsByTagName("RETENTION")[0];
    if (rtnObj) {
      var rtnArr = rtnObj.childNodes;
      if (rtnArr.length > 0) {
	Store["ST" + s + ".RETENTION"] = 1;
      }
      for (var i = 0; i < rtnArr.length; i++) {
	if (rtnArr[i].tagName) {
	  //	Notify("Purpose: <" + prpsObjArr[i].tagName + ">");
	  Store["ST" + s + ".RETENTION-" + rtnArr[i].tagName] = 1;
	}
      }
    }
    else {
      Store["ST" + s + ".RETENTION"] = 0;
    }

    // Parse DATA-GROUP
    var dataGrp = stmtArr[s].getElementsByTagName("DATA-GROUP")[0];
    if (dataGrp) {
      var dataArr = dataGrp.getElementsByTagName("DATA");
      Store["ST" + s + ".numData"] = dataArr.length;
      for (var i = 0; i < dataArr.length; i++) {
	//	Notify("Data ref: <" + dataArr[i].getAttribute("ref") + ">");
	var type = dataArr[i].getAttribute("ref");
	var typeStr = type.substr(1,type.length-1);
	var catObj = dataArr[i].getElementsByTagName("CATEGORIES")[0];
	if (catObj) { // CATEGORIES is optional
	  var catArr = catObj.childNodes;
	}
	var catStr = "";
	if (catArr) {
	  for (var c = 0; c < catArr.length; c++) {
	    if (catArr[c]) {
	      //		Notify("ST" + s + ".DATA-" + i + "." + typeStr + "+" + catArr[c].tagName);
	      catStr += catArr[c].tagName + "+";
	    }
	  }
	}
	if (dataArr[i].getAttribute("optional")) {
	  Store["ST" + s + ".DATA-" + i] = typeStr + "-" + catStr + "=" + "optional";
	}
	else {
	  Store["ST" + s + ".DATA-" + i] = typeStr + "-" + catStr + "=" + "notOptional";
	}
      } // end for data
    } // end if dataGrp
    else {
      Store["ST" + s + ".numData"] = 0;
    }

  } // end for loop throurgh statements

  Notify("From ParsePolicyFile, calling Translate on policy named " + Store.policyName);
  ShortNotice();
  DetailedTranslation();

}

/***************************************************************************
ShortNotice(): Creates a tab with a summarized privacy policy in tabular form.
****************************************************************************/
function ShortNotice() {

/*  var tb = getBrowser();
  var newtab = tb.addTab("chrome://privacyfox/content/template.html");
  var b = tb.getBrowserForTab(newtab);
  var doc = b.contentDocument;
  // do whatever you want to the document here...
  // doc.getElementById("something").value = "Some text";

  // Write page heading
  var elem = doc.createElement("h1");
  text = doc.createTextNode("Privacy Policy Summary - " + Store["business.name"]);  
  elem.appendChild(text);
  doc.body.appendChild(elem);
  doc.write("<h2>.Write Test</h2>");

  // write links to HR policy & full translation
  p = doc.createElement("p");
  text = doc.createTextNode("This is a summary of this site's privacy policy. For more details, ");
  p.appendChild(text);
  url = doc.createElement("a");
  url.setAttribute("href", "javascript:DetailedTranslation();");
  link = doc.createTextNode("read the full translation");
  url.appendChild(link);
  p.appendChild(url);
  text = doc.createTextNode(", or read the site's ");
  p.appendChild(text);  
  url = doc.createElement("a");
  url.setAttribute("href", Store.discURI);
  link = doc.createTextNode("own, full privacy policy");
  url.appendChild(link);
  p.appendChild(url);
  text = doc.createTextNode(".");
  p.appendChild(text);
  doc.body.appendChild(p);
  tb.selectedTab = newtab; 
*/
  var browser = getBrowser();
  var tab = browser.addTab("");
  var b = browser.getBrowserForTab(tab);
  var mydoc = b.contentDocument;
/*  var tab = gBrowser.addTab("");
  var mydoc = gBrowser.getBrowserForTab(tab).contentDocument;
*/
  // I have to use doc.write() to create write the body into the tab first. Otherwise, I can't use DOM to append elements
  // to the body. An alternate is to load a blank page in and use it instead. I was having trouble with accessing 
  // .css & .js files sitting in the chrome/content/privacyfox folder from within the template. See the link src=chrome:
  // line below.
  mydoc.write("<html>\n<head>\n" + 
	      "<title>Privacy Policy Summary - " + Store["business.name"] + "</title>\n" +
	      //	      "<link rel=\"stylesheet\" type=\"text/css\" src=\"chrome://content/privacyfox/privacyfox.css\">\n" +
	      "<style  type=\"text/css\">\n" +
	      "body {   font-family: Tahoma, Geneva, Arial, Helvetica, sans-serif;  font-size: 14px;  background-color: #EEEEEE;  vertical-align: top;  /*  text-align: center; */}\n" +
	      "a {   color: #0099FF; }  a:hover {  color: #336699; }  a:visited {   color: #666666; text-align: center; }\n" +
	      "table {  font-family: Tahoma, Geneva, Arial, Helvetica, sans-serif;  font-size: 14px; } " +
	      "thead {  background-color: #EFEFEF;   font-size: 15px;  font-weight: bold;   text-align: center;} " + 
	      "tbody {  vertical-align: top; text-align: center; }\n" +
	      "ul {  text-align: left; font-size: 13px; } \n </style>\n" +
	      "</head>\n\n<body> </body>\n</html>");
  mydoc.close();

  // Write page heading
  var elem = mydoc.createElement("h1");
  text = mydoc.createTextNode("Privacy Policy Summary - " + Store["business.name"]);  
  elem.appendChild(text);
  mydoc.body.appendChild(elem);

  // write links to HR policy & full translation
  p = mydoc.createElement("p");
  text = mydoc.createTextNode("This is a summary of this site's privacy policy. For more details, ");
  p.appendChild(text);
  url = mydoc.createElement("a");
  url.setAttribute("href", " ");
  link = mydoc.createTextNode("read the Privacy Fox's detailed translation");
  url.appendChild(link);
  p.appendChild(url);
  text = mydoc.createTextNode(", or read the site's ");
  p.appendChild(text);  
  url = mydoc.createElement("a");
  url.setAttribute("href", Store.discURI);
  link = mydoc.createTextNode("own, human-readable privacy policy.");
  url.appendChild(link);
  p.appendChild(url);
  text = mydoc.createTextNode(".");
  p.appendChild(text);
  mydoc.body.appendChild(p);

  table = mydoc.createElement("table");
  table.setAttribute("border", "1");
  table.setAttribute("align", "center");
  thead = mydoc.createElement("thead");
  tr = mydoc.createElement("tr");

  td = mydoc.createElement("td");
  td.setAttribute("width", "30%");
  text = mydoc.createTextNode("What data do we collect?");
  td.appendChild(text);
  tr.appendChild(td);

  td = mydoc.createElement("td");
  td.setAttribute("width", "40%");
  text = mydoc.createTextNode("Why do we collect this data?");
  td.appendChild(text);
  tr.appendChild(td);
  
  td = mydoc.createElement("td");
  td.setAttribute("width", "20%");
  text = mydoc.createTextNode("Can we personally identify you from this data?");
  td.appendChild(text);
  tr.appendChild(td);

  td = mydoc.createElement("td");
  td.setAttribute("width", "10%");
  text = mydoc.createTextNode("Who can see this data?");
  td.appendChild(text);
  tr.appendChild(td); 

  thead.appendChild(tr);
  table.appendChild(thead);
  mydoc.body.appendChild(table);

  tbody = mydoc.createElement("tbody");
  if (Store["numST"] ==  0) {
    tr = mydoc.createElement("tr");
    td = mydoc.createElement("td"); 
    td.setAttribute("colspan", "4");
    h3 = mydoc.createElement("h3");
    font = mydoc.createElement("font");    
    font.setAttribute("color", "red");
    text = mydoc.createTextNode("This statement gives you no information about what data it collects, who it is shared with,  and how it is used!");
    font.appendChild(text);
    h4.appendChild(font);
    td.appendChild(h3);
    td.appendChild(td);
  }

  //Notify("Before for loop");
  for (var s = 0; s < Store["numST"]; s++) {
    //Notify("Just entered for loop.");
    var st = "ST" + s;
    tr = mydoc.createElement("tr"); 
    // each statement has a row composed of four cells
    
    // column for data collected type
    td = mydoc.createElement("td");
    //  text = mydoc.createTextNode("Placeholder for types of data collected.");
    //Notify("Just before if nonident");
 /*   if (Store[st + ".NON-IDENTIFIABLE"]) {
      text = mydoc.createTextNode("We do not keep any information that could be used " + 
				  "to identify you personally.\n");
      td.appendChild(text);
    }
    else 
 */
    if (Store[st +".numData"] > 0) {
      //Notify("Just into if numdata");    
      var size = Store[st +".numData"];
      ul = mydoc.createElement("ul");
      // Notify("Just before for d");
      for (var d = 0; d < size; d++) {
	// Notify("In for d. d = " + d);
	li = mydoc.createElement("li");
	var data = Store[st +".DATA-" + d];
	var regexpDyn = /dynamic.miscdata-(\w+)\W+(\w+ptional)/;
	if (regexpDyn.test(data)){ // this is categories
	  // text = mydoc.createTextNode(RegExp.$1);
	  var cat = RegExp.$1;
	  var opt = "";
	  if (RegExp.$2 == "notOptional") // only mark not optional data as such
	    opt = "Not Optional: ";

	  if (cat == "physical") 
	    text = mydoc.createTextNode(opt + "Physical contact info.");
	  else if (cat == "online") 
	    text = mydoc.createTextNode(opt + "Online contact info.");
	else if (cat == "uniqueid")
	  text = mydoc.createTextNode(opt + "Website login IDs and other unique identifiers (not govt.-issued or financial");
	else if (cat == "purchase")
	  text = mydoc.createTextNode(opt + "Info. about your purchases & payment methods");
	else if (cat == "financial") 
	    text = mydoc.createTextNode(opt + "Financial info. such as accounts, balances, and transaction history"); 
	else if (cat == "computer") 
	    text = mydoc.createTextNode(opt + "Info. about your computer and Internet connection"); 
	else if (cat == "navigation") 
	    text = mydoc.createTextNode(opt + "Pages you visited on this web site and how long you stayed at each page"); 
	else if (cat == "interactive") 
	    text = mydoc.createTextNode(opt + "Activities you engaged in at this web site, such as your searches and transactions"); 
	else if (cat == "demographic") 
	    text = mydoc.createTextNode(opt + "Demographical info."); 
	else if (cat == "content") 
	    text = mydoc.createTextNode(opt + "Communication between you and the site");
	else if (cat == "state") 
	    text = mydoc.createTextNode(opt + "Cookies"); 
	else if (cat == "political") 
	    text = mydoc.createTextNode(opt + "Political affiliation"); 
	else if (cat == "health") 
	    text = mydoc.createTextNode(opt + "Your medical condition and health interests"); 
	else if (cat == "preference") 
	    text = mydoc.createTextNode(opt + "Your tastes or interests"); 
	else if (cat == "location") 
	    text = mydoc.createTextNode(opt + "Your exact geographic location"); 
	else if (cat == "government") 
	    text = mydoc.createTextNode(opt + "Government-issued identifiers such as S.S. numbers"); 
	else if (cat == "other-category") 
	  text = mydoc.createTextNode(opt + "Other types of data");
	else
	  text = mydoc.createTextNode("UNRECOGNIZED CATEGORY FOUND");
	} // end if test()
	// else if data is specific
	else { // fail-safe. Even if you don't understand the output, print it.
	  data = data.replace(/\./, " - "); // replace . in specific schema by -
	  data = data.replace(/\+/, "");
	  data = data.replace(/\-/g, "");
	  data = data.replace(/=.+$/, ""); // (TEMP) delete optional/nonoptional. Should fix this to prepending using regexp & backref
	  text = mydoc.createTextNode(data);
	}
	li.appendChild(text);
	ul.appendChild(li);
      } // end for d
      // Notify("Done with for d");  
      td.appendChild(ul);
    } // end if Store[numData]
    else {
      text = mydoc.createTextNode("None");
      td.appendChild(text);
    }
    tr.appendChild(td); // attach contents of data collected column to table

    // column for data usage
    td = mydoc.createElement("td");

    // don't need an else; the for loop takes care of 0 statements
    /*
      if ( Store[st + ".NON-IDENTIFIABLE"] ) {
      text = mydoc.createTextNode("-\n");
      td.appendChild(text);
      }
      else
    */ 
      if (Store[st + ".PURPOSE"]) {
      ul = mydoc.createElement("ul");

      if (Store[st + ".PURPOSE-telemarketing"]) {
	li = mydoc.createElement("li");
	text = mydoc.createTextNode("Telemarketing\n");
	li.appendChild(text);
	ul.appendChild(li);
      }
      if (Store[st + ".PURPOSE-contact"]) {
	li = mydoc.createElement("li");
	text = mydoc.createTextNode("Marketing via email or postal mail\n");
	li.appendChild(text);
	ul.appendChild(li);
      }
      if (Store[st + ".PURPOSE-admin"]) {
	li = mydoc.createElement("li");
	text = mydoc.createTextNode("Web site and system administration\n");
	li.appendChild(text);
	ul.appendChild(li);	  
      }	
      if (Store[st + ".PURPOSE-develop"]) {
	li = mydoc.createElement("li");
	text = mydoc.createTextNode("Anonymous R&D");
	li.appendChild(text);
	ul.appendChild(li);  
      }
      if ( (Store[st + ".PURPOSE-pseudo-decision"]) || (Store[st + ".PURPOSE-pseudo-analysis"]) ) {
	li = mydoc.createElement("li");
	text = mydoc.createTextNode("For pseudonymous profiling");
	li.appendChild(text);
	  ul.appendChild(li);
      }
      if ( (Store[st + ".PURPOSE-individual-analysis"]) || (Store[st + ".PURPOSE-individual-decision"]) ) {
	li = mydoc.createElement("li");
	text = mydoc.createTextNode("To profile you individually");
	li.appendChild(text);
	ul.appendChild(li);
      }
      if (Store[st + ".PURPOSE-other-purpose"]) {
	li = mydoc.createElement("li");
	if (Store[st + ".other-purpose"]) {
	  text = mydoc.createTextNode("For another purpose");
	}
	else {	    
	  text = mydoc.createTextNode("For an unspecified other purpose");
	}
	li.appendChild(text);
	ul.appendChild(li);
      } // end if other purpose
      
      td.appendChild(ul);
      
    } // end if Store[st+PURPOSE]
    else { // if no purpose
      h4 = mydoc.createElement("h4");
      font = mydoc.createElement("font");    
      font.setAttribute("color", "red");
      text = mydoc.createTextNode("This statement gives you no information about how it uses the data it collects!");
      font.appendChild(text);
      h4.appendChild(font);
      td.appendChild(h4);
    }
    tr.appendChild(td); // append cell with purpose info
  
    //  column for personal identification from data
    td = mydoc.createElement("td");
    if ( (Store[st + ".PURPOSE-telemarketing"]) || 
	 (Store[st + ".PURPOSE-contact"]) ||
	 (Store[st + ".PURPOSE-individual-analysis"]) || 
	 (Store[st + ".PURPOSE-individual-decision"])  ) {
      text = mydoc.createTextNode("Yes\n");
    }
    else if (Store[st + ".NON-IDENTIFIABLE"]) {
      text = mydoc.createTextNode("No\n");
    }
    else {
      text = mydoc.createTextNode("Can't be determined automatically.\n");
    }
    td.appendChild(text);
    tr.appendChild(td); // append cell with PI info

    // sharing info
    td = mydoc.createElement("td");
    var stR = st + ".RECEPIENT";
    /*
      if ( Store[st + ".NON-IDENTIFIABLE"] ) {
      text = mydoc.createTextNode("-\n");
      td.appendChild(text);
      }
      else 
    */
    if (Store[stR] == 1) {
      if ( (Store[stR + "-ours"]) || (Store[stR + "-delivery"])  || (Store[stR + "-same"]) ) {
	text = mydoc.createTextNode("Us\n");
	p = mydoc.createElement("p");
	p.appendChild(text);
	td.appendChild(p);
      }
      if ((Store[stR + "-other-recepient"]) || (Store[stR + "-unrelated"]) ) {
	text = mydoc.createTextNode("Others\n");
	p = mydoc.createElement("p");
	p.appendChild(text);
	td.appendChild(p);     
      }
      if (Store[stR + "-public"]) {
	text = mydoc.createTextNode("Public\n");
	p = mydoc.createElement("p");
	p.appendChild(text);
	td.appendChild(p);     
      }
    } // end if ST#.RECEPIENT
    else {
      h4 = mydoc.createElement("h4");
      font = mydoc.createElement("font");    
      font.setAttribute("color", "red");
      text = mydoc.createTextNode("This policy does not tell you who your information is shared with.");
      font.appendChild(text);
      h4.appendChild(font);
      td.appendChild(h4);
    }
    tr.appendChild(td); // add sharing info

    tbody.appendChild(tr);
  } // end for s -> Store[numST]
  table.appendChild(tbody);

  browser.selectedTab = tab; // switch to short notice
}

/***************************************************************************
  Translate: creates a new window and translates the policy into it.
****************************************************************************/
function DetailedTranslation() {

  //  document.getElementById("writeroot").appendChild(document.createElement("hr"));
  Notify("Here's what I've parsed and stored to date for policy " + Store.policyName);
  for (var key in Store) {
    Notify("&nbsp;&nbsp;&nbsp;" + key + " => " + Store[key]);
  }
  
  var HTML = "<html><head><title>Privacy Policy Translation - " + Store["business.name"] + "</title>\n";
  HTML += "<style  type=\"text/css\">\n" +
    "body {   font-family: Tahoma, Geneva, Arial, Helvetica, sans-serif;  font-size: 12px;  background-color: #EEEEEE;  vertical-align: top;  /*  text-align: center; */}\n" +
    "a {   color: #0099FF; }  a:hover {  color: #336699; }  a:visited {   color: #666666; text-align: center; }\n" +
    "table {  font-family: Tahoma, Geneva, Arial, Helvetica, sans-serif;  font-size: 12px; } " +
    "thead {  background-color: #EFEFEF;   font-size: 13px;  font-weight: bold;   text-align: center;} " + 
    "tbody {  vertical-align: top; text-align: center; }\n" +
    "ul {  text-align: left; font-size: 11px; } \n </style>\n";
  //    "<style type=\"text/css\">\n<!--\ntd {\n\tvertical-align: top;\n}\n-->\n</style>\n"
  HTML += "</head>\n<body>\n";

  HTML += "<h1>" + Store["business.name"] + " - Privacy Policy</h1>\n"
    + "<p>Read our full privacy policy at <a href=\"" + Store["discURI"] 
    + "\" target=\"_blank\">" + Store["discURI"] + "</a>.\n" ;

  if (Store["optURI"] ) {
    HTML += " Find out how to opt-in or opt-out at <a href=\"" + Store["optURI"] 
      + "\" target=\"_blank\">" + Store["optURI"] + "</a>.</p>\n";
  }
  else if (!Store["ST0.PURPOSE"]){
    HTML += "<font color=\"red\">This policy does not tell you how to opt-in or opt-out.</font></p>\n";
  }

  if (Store["TEST"]) {
    HTML += "<h2>This is a test policy.</h2>\n";
  }

  if (Store["business.name"]) {
    HTML += "<p><strong>This policy is issued by:</strong></p>\n<address>";
    HTML += Store["business.name"];
    if (Store["business.contact-info.postal.street"]) {
      HTML += "\n<br>" + Store["business.contact-info.postal.street"];
    }
    if ( Store["business.contact-info.postal.city"] ) {
      HTML += "\n<br>" + Store["business.contact-info.postal.city"] + ", " ;
    }
    if (Store["business.contact-info.postal.stateprov"]) {
      HTML += Store["business.contact-info.postal.stateprov"] + " ";
    }
    if ( Store["business.contact-info.postal.postalcode"]) {
      HTML += Store["business.contact-info.postal.postalcode"] + ", ";
    }
    if (Store["business.contact-info.postal.country"]) {
      HTML += Store["business.contact-info.postal.country"];
    }
    if (Store["business.contact-info.online.uri"]) {
      HTML += "<br><a href=\"" + Store["business.contact-info.online.uri"] + "\">"
	+  Store["business.contact-info.online.uri"] + "</a>\n";
    }    
    if (Store["business.contact-info.online.email"]) {
      HTML += "<br><a href=\"mailto:" + Store["business.contact-info.online.email"] + "\">"
	+  Store["business.contact-info.online.email"] + "</a>\n";
    }
    HTML += "</address>\n";
  }
  else {
    HTML += "<p><font color=\"red\">This policy does not include any contact information about its publisher.</font></p>\n";
  }
  
  if (!Store["ACCESS"]) {
    HTML += "<p><font color=\"red\">This policy does not tell you what access it provides to information about you!</font>";
  }
  else {
    HTML += "<p><strong>Your access to information about you</strong>:\n";
  
    if (Store["ACCESS-nonident"] == 1){
      HTML += "We do not keep any information identified with you.\n";
    }
    else if (Store["ACCESS-all"] == "1") {
      HTML +=  "We give you access to all of our information identified with you.\n ";
    }
    else if (Store["ACCESS-contact-and-other"] == "1"){ 
      HTML += "We give you access to your contact information and some of our other information identified with you.\n";
    }
    else if (Store["ACCESS-ident-contact"] == "1") {
      HTML += "We give you access to only your contact information in our records.\n";
    }
    else if (Store["ACCESS-other-ident"] == "1") {
      HTML += "We allow you to access some of our information identified with you, but not your contact information.\n";
    }
    else if (Store["ACCESS-none"] == "1") {
      HTML += "We do not give you access to our information about you.\n";
    }
  } // end else !Store.ACCESS

  // DISPUTES section
  if (Store["DISPUTES-GROUP"]) {
    if ( Store["DISPUTES-service"] || Store["DISPUTES-independent"] ) {
      HTML += "<p><strong>Ways to resolve privacy-related disputes with us include:</strong></p>\n<ul>";
    }
    if (Store["DISPUTES-service"] == 1) { //disputes are solved through customer service
      if (Store["DISPUTES-service-sDesc"]) {
	HTML += "<li><a href=\"" + Store["DISPUTES-service-service"] + "\" target=\"_blank\">" 
	  + Store["DISPUTES-service-sDesc"] + "</a>: \n";
      }
      else { // short description is absent - revert to P3P spec default
	HTML += "<li><a href=\"" + Store["DISPUTES-service-service"] 
	  + "\" target=\"_blank\">Customer Service</a>: \n";
      }
      if (Store["DISPUTES-service-lDesc"]) {
	HTML += " " + Store["DISPUTES-service-lDesc"];     
      }
      var x = 1;
      if (Store["DISPUTES-service-rem-correct"]) {
	HTML += " We will correct any errors we make related to the commitments in our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-service-rem-money"]) {
	HTML += " We will compensate individuals if it is determined that we have violated our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-service-rem-law"]) {
	HTML += " Our privacy policy references a law that may determine remedies for breaches of our policy.";
	x = 0;
      }
      if (x) { // no remedies have been provided with this dispute
	HTML += " <font color=\"red\">This dispute section provides no remedies for privacy-related breaches!</font>";
      }
    }

    if (Store["DISPUTES-independent"] == 1) { // disputes are solved through an independent org
      if (Store["DISPUTES-independent-sDesc"]) {
	HTML += "<li><a href=\"" + Store["DISPUTES-independent-service"] + "\" target=\"_blank\">" 
	  + Store["DISPUTES-independent-sDesc"] + "</a>: \n";
      }
      else { // short description is absent - revert to P3P spec default
	HTML += "<li><a href=\"" + Store["DISPUTES-independent-service"] 
	  + "\" target=\"_blank\">Independent Organization</a>: \n";
      }
      if (Store["DISPUTES-independent-lDesc"]) {
	HTML += " " + Store["DISPUTES-independent-lDesc"];     
      }
      if (Store["DISPUTES-independent-verify"]) {
	HTML += "<a href=\"" + Store["DISPUTES-independent-service"] 
	  + "\" target=\"_blank\">Verify</a>";
      }
      var x = 1;
      if (Store["DISPUTES-independent-rem-correct"]) {
	HTML += " We will correct any errors we make related to the commitments in our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-independent-rem-money"]) {
	HTML += " We will compensate individuals if it is determined that we have violated our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-independent-rem-law"]) {
	HTML += " Our privacy policy references a law that may determine remedies for breaches of our policy.";
	x = 0;
      }
      if (x) { // no remedies have been provided with this dispute
	HTML += " <font color=\"red\">This dispute section  provides no remedies for privacy-related breaches!</font>";
      }
    }

    if (Store["DISPUTES-service"] || Store["DISPUTES-independent"]) {
      HTML += "</ul>\n";
    }

    if (Store["DISPUTES-court"] == 1) { // courts have jurisdiction over disputes
      HTML += "<p>We believe that the following authority offers recourse for disputes: \n";
      if (Store["DISPUTES-court-sDesc"]) {
	HTML += "<a href=\"" + Store["DISPUTES-court-service"] + "\" target=\"_blank\">" 
	  + Store["DISPUTES-court-sDesc"] + "</a>: ";
      }
      else { // short description is absent - revert to P3P spec default
	HTML += "<li><a href=\"" + Store["DISPUTES-court-service"] 
	  + "\" target=\"_blank\">Possbile legal complaint</a>: ";
      }
      if (Store["DISPUTES-court-lDesc"]) {
	HTML += " " + Store["DISPUTES-court-lDesc"];     
      }
      var x = 1;
      if (Store["DISPUTES-court-rem-correct"]) {
	HTML += " We will correct any errors we make related to the commitments in our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-court-rem-money"]) {
	HTML += " We will compensate individuals if it is determined that we have violated our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-court-rem-law"]) {
	HTML += " Our privacy policy references a law that may determine remedies for breaches of our policy.";
	x = 0;
      }
      if (x) { // no remedies have been provided with this dispute
	HTML += " <font color=\"red\">This dispute section provides no remedies for privacy-related breaches!</font>";
      }
    }
    
    if (Store["DISPUTES-law"] == 1) { // law/regulation covers disputes
      HTML += "<p>We believe that the following laws or regulations provide recourse: \n";
      if (Store["DISPUTES-law-sDesc"]) {
	HTML += "<li><a href=\"" + Store["DISPUTES-law-service"] + "\" target=\"_blank\">" 
	  + Store["DISPUTES-law-sDesc"] + "</a>: ";
      }
      else { // short description is absent - revert to P3P spec default
	HTML += "<li><a href=\"" + Store["DISPUTES-law-service"] 
	  + "\" target=\"_blank\">Law</a>: ";
      }
      if (Store["DISPUTES-law-lDesc"]) {
	HTML += " " + Store["DISPUTES-law-lDesc"];     
      }
      var x = 1;
      if (Store["DISPUTES-law-rem-correct"]) {
	HTML += " We will correct any errors we make related to the commitments in our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-law-rem-money"]) {
	HTML += " We will compensate individuals if it is determined that we have violated our privacy policy.";
	x = 0;
      }
      if (Store["DISPUTES-law-rem-law"]) {
	HTML += " Our privacy policy references a law that may determine remedies for breaches of our policy.";
	x = 0;
      }
      if (x) { // no remedies have been provided with this dispute
	HTML += " <font color=\"red\">This dispute section provides no remedies for privacy-related breaches!</font>";
      }
    }
  } // end if DISPUTES-GROUP
  else if (!Store["ST0.NON-IDENTIFIABLE"]) {
    HTML += "<p>This statement provides no means for resolving privacy-related disputes (but it does not claim to collect personally identifiable information).</p>\n";
  }
  else {
    HTML += "<p><font color=\"red\">This statement provides no means for resolving privacy-related disputes!</font></p>\n";   
  } // end else if disputes

  // translate statements
  if (Store["numST"] ==  0) {
    HTML += "<h3><font color=\"red\">This statement gives you no information about what data it collects and how it is used!</font></h3>\n";
  }
/*  else {
    //    HTML += "<p>Statement # " + Store["numST"] + "</p>\n";

  }
*/
  HTML += "<table border=\"1\" valign=\"top\">\n";
  // don't need an else; the for loop takes care of 0 statements
  for (var s = 0; s < Store["numST"]; s++) {
    var st = "ST" + s;
    HTML += "<tr><td colspan=\"3\" bgcolor=\"#CCCCCC\"><h3>Statement # " + (s+1) + "</h3>";
    if (Store[st + ".CONSEQUENCE"]) {
      HTML += "<p><strong>Summary</strong>: \n" + Store[st + ".CONSEQUENCE"] 
	+ "</p>\n";
    }
    if (Store[st + ".NON-IDENTIFIABLE"]) {
      HTML += "<h4>We do not keep any information that could be used to identify you personally.</h4>";
    }
    // translate RETENTION
    stR = st + ".RETENTION";
    if (Store[stR] == 1) {
      HTML += "<p><strong>How long we may keep your information:</strong> ";
      if (Store[stR + "-no-retention"]) {
	HTML += "<We do not keep your information beyond your current online session.</p>\n";
      }
      else if (Store[stR + "-stated-purpose"]) {
	HTML += "We keep your information only long enough to perform the activity for which we collected it.</p>\n";
      }    
      else if (Store[stR + "-legal-requirement"]) {
	HTML += "We keep your information only as long as we need to for legal purposes.</p>\n";
      }
      else if (Store[stR + "-business-practices"]) {
	HTML += "Our full privacy policy explains how long we keep your information.</p>\n";
      }
      else if (Store[stR + "-indefinitely"]) {
	HTML += "We may keep your information indefinitely.</p>\n";
      }
    }
    else {
      HTML += "<h4><font color=\"red\">This policy does not tell you how long your information will be kept.</h4>\n";
    } // end if stR (retention)
    HTML += "\n</td>\n<tr>\n";
    
    // DATA COLLECTED    
    HTML += "<td width=\"34%\">\n";    // column for data collected type
    if (Store[st + ".NON-IDENTIFIABLE"]) {
      HTML +="<h4>We do not collect any information that could be used to identify you personally.</h4>\n";
      // td.appendChild(text);
    }
    else if (Store[st +".numData"] > 0) {
      //Notify("Just into if numdata");    
      var size = Store[st +".numData"];
      HTML += "<h4>We may collect the following types of information about you:</h4>\n<ul>\n";
      // ul = mydoc.createElement("ul");
      // Notify("Just before for d");
      for (var d = 0; d < size; d++) {
	// Notify("In for d. d = " + d);
	HTML += "<li>";
	// li = mydoc.createElement("li");
	var data = Store[st +".DATA-" + d];
	var regexpDyn = /dynamic.miscdata-(\w+)\W+(\w+ptional)/;
	if (regexpDyn.test(data)){ // this is categories
	  // text = mydoc.createTextNode(RegExp.$1);
	  var cat = RegExp.$1;
	  var opt = "";
	  if (RegExp.$2 == "notOptional") // only mark not optional data as such
	    opt = "Not Optional: ";
	  
	  if (cat == "physical") 
	    HTML += opt + "Name, address, phone number, or other physical contact information";
	  else if (cat == "online") 
	    HTML += opt + "Email address or other online contact information";
	  else if (cat == "uniqueid")
	    HTML += opt + "Website login IDs and other identifiers (excluding government IDs and financial account numbers";
	  else if (cat == "purchase")
	    HTML += opt + "Information about your purchases, including payment methods";
	  else if (cat == "financial") 
	    HTML += opt + "Financial information such as accounts, balances, and transaction history"; 
	  else if (cat == "computer")
	    HTML += opt + "Information about the computer you are using, such as its hardware, software, or Internet address"; 
	  else if (cat == "navigation") 
	    HTML += opt + "Which pages you visited on this web site and how long you stayed at each page"; 
	  else if (cat == "interactive") 
	    HTML += opt + "Activities you engaged in at this web site, such as your searches and transactions"; 
	  else if (cat == "demographic") 
	    HTML += opt + "Information about social and economic categories that might apply to you, such as your gender, age, income, or where you are from"; 
	  else if (cat == "content") 
	    HTML += opt + "Messages you send to us or post on this site, such as email, bulletin board postings, or chat room conversations";
	  else if (cat == "state") 
	    HTML += opt + "Cookies and mechanisms that perform similar functions"; 
	  else if (cat == "political") 
	    HTML += opt + "Which groups you might be a member of such as religious organizations, trade unions, and political parties"; 
	  else if (cat == "health") 
	    HTML += opt + "Health information such as information about your medical condition or your interest in health-related topics, services, or products"; 
	  else if (cat == "preference") 
	    HTML += opt + "Information about your tastes or interests"; 
	  else if (cat == "location") 
	    HTML += opt + "Information about an exact geographic location, such as data transmitted by your GPS-enabled device"; 
	  else if (cat == "government") 
	    HTML += opt + "Government-issued identifiers such as social security numbers"; 
	  else if (cat == "other-category") 
	    HTML += opt + "Other types of data: include site's human, readable explanation; if site omits human-readable explanation say \"not described here\"";
	  else
	    HTML += "UNRECOGNIZED CATEGORY FOUND";
	} // end if test()
	else { // if data is specific, fail-safe. Even if you don't understand the output, print it.
	  data = data.replace(/\./, " - "); // replace . in specific schema by -
	  data = data.replace(/\+/, "");
	  data = data.replace(/\-/g, "");
	  data = data.replace(/=.+$/, ""); // (TEMP) delete optional/nonoptional. Should fix this to prepending using regexp & backref
	  HTML += data;
	}
	HTML += "</li>\n";
	// li.appendChild(text);
	// ul.appendChild(li);
	
      } // end for d (data elements)
      // Notify("Done with for d");
      HTML += "</ul>\n</td>\n";
      // td.appendChild(ul);
    } // end if Store[numData]
    
    
    if (Store[st + ".PURPOSE"]) {
      HTML += "<td width=\"33%\"><h4>This site may use your information in the following ways:</h4> <ul>\n";
      if (Store[st + ".PURPOSE-current"]) {
	HTML += "<li>To provide the service you requested</li>\n";
      }
      if (Store[st + ".PURPOSE-admin"]) {
	HTML += "<li>To perform web site and system administration </li>\n";
      }
      if (Store[st + ".PURPOSE-develop"]) {
	HTML += "<li>For research and development, but without connecting any information to you</li>\n";
      }
      if (Store[st + ".PURPOSE-tailoring"]) {
	HTML += "<li>To customize the site for your current visit only</li>\n";
      }
      if (Store[st + ".PURPOSE-pseudo-analysis"]) {
	HTML += "<li>To do research and analysis in which your information may be linked to an ID code but not to your personal identity </li>\n";
      }
      if (Store[st + ".PURPOSE-pseudo-decision"]) {
	HTML += "<li>To make decisions that directly affect you without identifying you, e.g., to display content or ads based on links you clicked on previously </li>\n";
      }
      if (Store[st + ".PURPOSE-individual-analysis"]) {
	HTML += "<li>To do research and analysis that uses information about you</li>\n";	
      }
      if (Store[st + ".PURPOSE-individual-decision"]) {
	HTML += "<li>To make decisions that directly affect you using information about you, e.g., to recommend products or services based on your previous purchases</li>\n";
      }
      if (Store[st + ".PURPOSE-contact"]) {
	HTML += "<li>To contact you through means other than telephone (e.g., email or postal mail) to market services or products</li>\n";
      }
      if (Store[st + ".PURPOSE-historical"]) {
	HTML += "<li>To aid in historical preservation as governed by a law or policy described in this privacy policy</li>\n";
      }
      if (Store[st + ".PURPOSE-telemarketing"]) {
	HTML += "<li>To contact you by telephone to market services or products</li>\n";
      }
      if (Store[st + ".PURPOSE-other-purpose"]) {
	HTML += "<li>For other uses: ";
	if (Store[st + ".other-purpose"]) {
	  HTML += Store[st + ".other-purpose"];
	}
	else {
	  HTML += "<strong><font color=\"red\">not described here</font></strong>";
	}
	HTML += "</li>\n";
      }
      HTML += "</ul>\n</td>\n";
    } // end if Store[st+PURPOSE]
    else {
      HTML += "<td><font color=\"red\">This policy does not tell you in what ways your information may be used.</font></td>";
    }
    
    // translate RECEPIENT
    var stR = st + ".RECEPIENT";
    if (Store[stR] == 1) {
      HTML += "<td width=\"33%\"><h4>We may share your information with:</h4>\n<ul>\n";
      if (Store[stR + "-ours"]) {
	HTML += "<li>Companies that help us fulfill your requests (for example, shipping a product to you), but these companies must not use your information for any other purpose.</li>\n";
      }
      if (Store[stR + "-delivery"]) {
	HTML += "<li>Delivery companies that help us fulfill your requests and who may also use your information in other ways.</li>\n";
      }    
      if (Store[stR + "-same"]) {
	HTML += "<li>Companies that have privacy policies similar to ours.</li>\n";
      }
      if (Store[stR + "-other-recepient"]) {
	HTML += "<li>Companies that are accountable to us, though their privacy policies may be different from ours.</li>\n";
      }
      if (Store[stR + "-unrelated"]) {
	HTML += "<li>Other companies whose privacy policies are unknown to us.</li>\n";
      }
      if (Store[stR + "-public"]) {
	HTML += "<li>People who may access your information from a public area, such as a bulletin board, chat room, or directory.</li>\n";
      }
      HTML += "</ul>\n</td>\n";
    }
    else {
      HTML += "<td><font color=\"red\">This policy does not tell you who your information is shared with.</font></td>\n";
    }
    // translate data types
    
    //  } // end else if NON-IDENTIFIABLE
    HTML += "</tr>\n";
  } // end for s 0 .. numStatements
  HTML += "</table>\n";
  
  HTML += "</body></html>";
  // MyWin.document.write(HTML);
  
  /*  
      var uri = document.implementation.createDocument("", "", null);
      var tab = gBrowser.addTab(uri);
      gBrowser.getBrowserForTab(tab).contentDocument;
      var text = tab.document.createTextNode(Store["business.name"] + " - Privacy Policy");  
      var elem = tab.document.createElement("h1");
      elem.appendChild(text);
      tab.document.body.appendChild(elem);
  */  
  var TransTab = gBrowser.addTab("");
  var TransDoc = gBrowser.getBrowserForTab(TransTab).contentDocument;
  TransDoc.write(HTML);
  TransDoc.close();
  Debug.close();
  
}
